<template>
  <div class="p-2">
    <el-card shadow="never">
      <template #header>
        <el-row :gutter="10" class="mb8">
          <el-col :span="1.5">
            <el-button type="primary" plain icon="Plus" @click="handleAdd" v-hasPermi="['qms:testResult:add']">新增</el-button>
          </el-col>
          <el-col :span="1.5">
            <el-button type="success" plain icon="Edit" :disabled="single" @click="handleUpdate()" v-hasPermi="['qms:testResult:edit']">修改</el-button>
          </el-col>
          <el-col :span="1.5">
            <el-button type="danger" plain icon="Delete" :disabled="multiple" @click="handleDelete()" v-hasPermi="['qms:testResult:remove']">删除</el-button>
          </el-col>
          <el-col :span="1.5">
            <el-button type="warning" plain icon="Download" @click="handleExport" v-hasPermi="['qms:testResult:export']">导出</el-button>
          </el-col>

          <right-toolbar :columns="columns" v-model:showSearch="showSearch" @queryTable="getList"></right-toolbar>
        </el-row>
      </template>

      <el-table v-loading="loading" :data="testResultList" @selection-change="handleSelectionChange">
        <el-table-column type="selection" width="55" align="center" />
        <template v-for="col in visibleColumns" :key="col.prop">
          <el-table-column
            :label="col.label"
            :align="col.align"
            :prop="col.prop"
            v-if="col.visible"
          >
            <template #header="{ column }">
              <div @dblclick="handleColumnDoubleClick(column.property)">
                {{ column.label }}
              </div>
            </template>
          </el-table-column>
        </template>
        <el-table-column label="操作" align="center" class-name="small-padding fixed-width">
          <template #default="scope">
            <el-tooltip content="修改" placement="top">
              <el-button link type="primary" icon="Edit" @click="handleUpdate(scope.row)" v-hasPermi="['qms:testResult:edit']"></el-button>
            </el-tooltip>
            <el-tooltip content="删除" placement="top">
              <el-button link type="primary" icon="Delete" @click="handleDelete(scope.row)" v-hasPermi="['qms:testResult:remove']"></el-button>
            </el-tooltip>
          </template>
        </el-table-column>
      </el-table>

      <pagination v-show="total > 0" :total="total" v-model:page="queryParams.pageNum" v-model:limit="queryParams.pageSize" @pagination="getList" />
    </el-card>
    <!-- 添加或修改测试结果对话框 -->
    <el-dialog :title="dialog.title" v-model="dialog.visible" width="500px" append-to-body>
      <el-form ref="testResultFormRef" :model="form" :rules="rules" label-width="80px">
        <el-form-item label="批次号" prop="batchCode">
          <el-input v-model="form.batchCode" placeholder="请输入批次号" />
        </el-form-item>
        <el-form-item label="测试序号" prop="testNum">
          <el-input v-model="form.testNum" placeholder="请输入测试序号" />
        </el-form-item>
        <el-form-item label="测试项目" prop="testItem">
          <el-input v-model="form.testItem" placeholder="请输入测试项目" />
        </el-form-item>
        <el-form-item label="实际电压" prop="voltage">
          <el-input v-model="form.voltage" placeholder="请输入实际电压" />
        </el-form-item>
        <el-form-item label="实际电流" prop="loadCurrent">
          <el-input v-model="form.loadCurrent" placeholder="请输入实际电流" />
        </el-form-item>
        <el-form-item label="标准距离" prop="stdDistance">
          <el-input v-model="form.stdDistance" placeholder="请输入标准距离" />
        </el-form-item>
        <el-form-item label="感应物" prop="inductor">
          <el-input v-model="form.inductor" placeholder="请输入感应物" />
        </el-form-item>
        <el-form-item label="输出引脚" prop="output">
          <el-input v-model="form.output" placeholder="请输入输出引脚" />
        </el-form-item>
        <el-form-item label="测试数据" prop="testValue">
          <el-input v-model="form.testValue" placeholder="请输入测试数据" />
        </el-form-item>
        <el-form-item label="判断条件" prop="judgeDetail">
          <el-input v-model="form.judgeDetail" placeholder="请输入判断条件" />
        </el-form-item>
        <el-form-item label="测试结果" prop="testResult">
          <el-input v-model="form.testResult" placeholder="请输入测试结果" />
        </el-form-item>
        <el-form-item label="备注" prop="remark">
          <el-input v-model="form.remark" placeholder="请输入备注" />
        </el-form-item>
      </el-form>
      <template #footer>
        <div class="dialog-footer">
          <el-button :loading="buttonLoading" type="primary" @click="submitForm">确 定</el-button>
          <el-button @click="cancel">取 消</el-button>
        </div>
      </template>
    </el-dialog>
    <!-- 列选择对话框 -->
    <el-dialog title="选择列" v-model="showColumnSelector" width="300px" append-to-body @close="showColumnSelector = false">
      <el-checkbox-group v-model="selectedColumns">
        <el-checkbox v-for="column in columns" :key="column.prop" :label="column.prop">{{ column.label }}</el-checkbox>
      </el-checkbox-group>
      <template #footer>
        <div class="dialog-footer">
          <el-button type="primary" @click="saveColumnSelection">确 定</el-button>
          <el-button @click="showColumnSelector = false">取 消</el-button>
        </div>
      </template>
    </el-dialog>
  </div>
</template>

<script setup name="TestResult" lang="ts">
import { listTestResult, getTestResult, delTestResult, addTestResult, updateTestResult } from '@/api/qms/testResult';
import { TestResultVO, TestResultQuery, TestResultForm } from '@/api/qms/testResult/types';
import { ref, reactive, watch, onMounted, computed } from 'vue';

const { proxy } = getCurrentInstance() as ComponentInternalInstance;

const testResultList = ref<TestResultVO[]>([]);
const buttonLoading = ref(false);
const loading = ref(true);
const showSearch = ref(true);
const ids = ref<Array<string | number>>([]);
const single = ref(true);
const multiple = ref(true);
const total = ref(0);

const queryFormRef = ref<ElFormInstance>();
const testResultFormRef = ref<ElFormInstance>();

const dialog = reactive<DialogOption>({
  visible: false,
  title: ''
});

const initFormData: TestResultForm = {
  id: undefined,
  batchCode: undefined,
  testNum: undefined,
  testItem: undefined,
  voltage: undefined,
  loadCurrent: undefined,
  stdDistance: undefined,
  inductor: undefined,
  output: undefined,
  testValue: undefined,
  judgeDetail: undefined,
  testResult: undefined,
  remark: undefined
}
const data = reactive<PageData<TestResultForm, TestResultQuery>>({
  form: {...initFormData},
  queryParams: {
    pageNum: 1,
    pageSize: 20,
    batchCode: undefined,
    testNum: undefined,
    testItem: undefined,
    voltage: undefined,
    loadCurrent: undefined,
    stdDistance: undefined,
    inductor: undefined,
    output: undefined,
    testValue: undefined,
    judgeDetail: undefined,
    testResult: undefined,
    params: {
    }
  },
  rules: {
    id: [
      { required: true, message: "不能为空", trigger: "blur" }
    ],
    batchCode: [
      { required: true, message: "批次号不能为空", trigger: "blur" }
    ],
    testNum: [
      { required: true, message: "测试序号不能为空", trigger: "blur" }
    ],
    testItem: [
      { required: true, message: "测试项目不能为空", trigger: "blur" }
    ],
    voltage: [
      { required: true, message: "实际电压不能为空", trigger: "blur" }
    ],
    loadCurrent: [
      { required: true, message: "实际电流不能为空", trigger: "blur" }
    ],
    stdDistance: [
      { required: true, message: "标准距离不能为空", trigger: "blur" }
    ],
    inductor: [
      { required: true, message: "感应物不能为空", trigger: "blur" }
    ],
    output: [
      { required: true, message: "输出引脚不能为空", trigger: "blur" }
    ],
    testValue: [
      { required: true, message: "测试数据不能为空", trigger: "blur" }
    ],
    judgeDetail: [
      { required: true, message: "判断条件不能为空", trigger: "blur" }
    ],
    testResult: [
      { required: true, message: "测试结果不能为空", trigger: "blur" }
    ],
  }
});

const { queryParams, form, rules } = toRefs(data);

/** 查询测试结果列表 */
const getList = async () => {
  loading.value = true;
  queryParams.value.batchCode = proxy.$route.query.batchCode;
  queryParams.value.testNum = proxy.$route.query.testNum;

  const res = await listTestResult(queryParams.value);
  testResultList.value = res.rows;
  total.value = res.total;
  loading.value = false;
}

/** 取消按钮 */
const cancel = () => {
  reset();
  dialog.visible = false;
}

/** 表单重置 */
const reset = () => {
  form.value = {...initFormData};
  testResultFormRef.value?.resetFields();
}

/** 搜索按钮操作 */
const handleQuery = () => {
  queryParams.value.pageNum = 1;
  getList();
}

/** 重置按钮操作 */
const resetQuery = () => {
  queryFormRef.value?.resetFields();
  handleQuery();
}

/** 多选框选中数据 */
const handleSelectionChange = (selection: TestResultVO[]) => {
  ids.value = selection.map(item => item.id);
  single.value = selection.length != 1;
  multiple.value = !selection.length;
}

/** 新增按钮操作 */
const handleAdd = () => {
  reset();
  dialog.visible = true;
  dialog.title = "添加测试结果";
}

/** 修改按钮操作 */
const handleUpdate = async (row?: TestResultVO) => {
  reset();
  const _id = row?.id || ids.value[0]
  const res = await getTestResult(_id);
  Object.assign(form.value, res.data);
  dialog.visible = true;
  dialog.title = "修改测试结果";
}

/** 提交按钮 */
const submitForm = () => {
  testResultFormRef.value?.validate(async (valid: boolean) => {
    if (valid) {
      buttonLoading.value = true;
      if (form.value.id) {
        await updateTestResult(form.value).finally(() =>  buttonLoading.value = false);
      } else {
        await addTestResult(form.value).finally(() =>  buttonLoading.value = false);
      }
      proxy?.$modal.msgSuccess("操作成功");
      dialog.visible = false;
      await getList();
    }
  });
}

/** 删除按钮操作 */
const handleDelete = async (row?: TestResultVO) => {
  const _ids = row?.id || ids.value;
  await proxy?.$modal.confirm('是否确认删除测试结果编号为"' + _ids + '"的数据项？').finally(() => loading.value = false);
  await delTestResult(_ids);
  proxy?.$modal.msgSuccess("删除成功");
  await getList();
}

/** 导出按钮操作 */
const handleExport = () => {
  proxy?.download('qms/testResult/export', {
    ...queryParams.value
  }, `testResult_${new Date().getTime()}.xlsx`)
}

const columns = [
  { prop: 'id', label: 'ID', align: 'center', visible: false },
  { prop: 'batchCode', label: '批次号', align: 'center', visible: true },
  { prop: 'testNum', label: '测试序号', align: 'center', visible: true },
  { prop: 'testItem', label: '测试项目', align: 'center', visible: true },
  { prop: 'voltage', label: '实际电压', align: 'center', visible: true },
  { prop: 'loadCurrent', label: '实际电流', align: 'center', visible: true },
  { prop: 'stdDistance', label: '标准距离', align: 'center', visible: true },
  { prop: 'inductor', label: '感应物', align: 'center', visible: true },
  { prop: 'output', label: '输出引脚', align: 'center', visible: true },
  { prop: 'testValue', label: '测试数据', align: 'center', visible: true },
  { prop: 'judgeDetail', label: '判断条件', align: 'center', visible: true },
  { prop: 'testResult', label: '测试结果', align: 'center', visible: true },
  { prop: 'remark', label: '备注', align: 'center', visible: true }
];

const selectedColumns = ref(columns.map(column => column.prop));

const visibleColumns = computed(() => {
  // 添加依赖项，确保当 selectedColumns 变化时，visibleColumns 也会重新计算
  return columns.filter(column => selectedColumns.value.includes(column.prop));
});

// 新增 watch 监听器，监听 selectedColumns 的变化
watch(()=> selectedColumns.value, (newSelectedColumns) => {
  console.log("selectedColumns", selectedColumns.value)
  columns.forEach(column => {
    column.visible = newSelectedColumns.includes(column.prop);
  });
});

const handleColumnDoubleClick = (prop: string) => {
  // 新增双击事件处理函数
  console.log('双击列：', prop);
  const column = columns.find(col => col.prop === prop);
  console.log(column)
  if (column) {
    column.visible = !column.visible;
  }
  selectedColumns.value = columns.filter(col => col.visible).map(col => col.prop);
  // // 确保 columns 的 visible 属性同步更新
  // columns.forEach(col => {
  //   col.visible = selectedColumns.value.includes(col.prop);
  // });
};

const saveColumnSelection = () => {
  columns.forEach(column => {
    column.visible = selectedColumns.value.includes(column.prop);
  });
};

watch(() => proxy.$route.query.testNum, (testNum) => {
  console.log("testNum", testNum)
  if(testNum) {
    queryParams.value.pageNum = 1;
    getList();
  }

})

onMounted(() => {
  getList();
});
</script>
